@@ -1,6 +1,6 @@
 (ns com.puppetlabs.puppetdb.http
-  (:require [ring.util.response :as rr])
-  (:use [com.puppetlabs.http :only [json-response]]))
+  (:require [ring.util.response :as rr]
+            [com.puppetlabs.http :refer [json-response]]))
 
 (def header-map
   "Maps the legal keys from a puppetdb query response object to the
@@ -36,3 +36,15 @@
   (->
     (json-response (:result query-result))
     (add-headers (dissoc query-result :result))))
+
+(defn remove-status
+  "Status is only for the v4 version of the reports response"
+  [result-map version]
+  (if-not (= :v4 version)
+    (dissoc result-map :status)
+    result-map))
+
+(defn v4?
+  "Returns a function that always returns true if `version` is :v4"
+  [version]
+  (constantly (= :v4 version)))
